package com.bmlaurus.alfresco;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.FileChannel.MapMode;
import java.util.ArrayList;
import java.util.List;

import org.alfresco.webservice.types.Reference;

import com.bmlaurus.alfresco.content.AlfrescoContentService;

/**
 * @author Antonio Caaveral
 * @version 1.0
 * @company BMLaurus
 * @email info@bmlaurus.com
 * */
public class AlfrescoUpload extends AlfrescoContentService {

  public AlfrescoUpload() {

  }

  public AlfrescoUpload(String server_host, String server_port, String server_username,
      String server_password) {
    this.server_host = server_host;
    this.server_port = server_port;
    this.server_username = server_username;
    this.server_password = server_password;
  }

  public void setServer_host(String server_host) {
    this.server_host = server_host;
  }

  public void setServer_port(String server_port) {
    this.server_port = server_port;
  }

  public void setServer_username(String server_username) {
    this.server_username = server_username;
  }

  public void setServer_password(String server_password) {
    this.server_password = server_password;
  }

  public void setContentDescription(String description) {
    content_description = description;
  }

  public void setContentTitle(String title) {
    content_title = title;
  }

  public void setContentAuthor(String author) {
    content_author = author;
  }

  public void setSpaceDescription(String description) {
    space_description = description;
  }

  public Reference createPath(String path) {
    startSession();
    String alfresco_path[] = path.split(SEPARATOR);
    Reference parent = null;
    try {
      parent = getCompanyHome();
      String currentNode = "";
      for (String unitPath : alfresco_path) {
        currentNode = currentNode + SEPARATOR + unitPath;
        Reference node = getSpaceReference(currentNode);
        if (node != null && node.getUuid() == null) {
          parent = createSpace(parent, unitPath);
        }
      }

    } catch (Exception e) {
      System.err.println("Error Creating the Alfresco Path");
      parent = null;
    }
    endSession();
    return parent;
  }

  private Reference createPathToAlfrescoNoSession(String path) {
    String alfresco_path[] = path.split(SEPARATOR);
    Reference parent = null;
    try {
      parent = getCompanyHome();
      String currentNode = "";
      for (String unitPath : alfresco_path) {
        currentNode = currentNode + SEPARATOR + unitPath;
        Reference node = getSpaceReference(currentNode);
        if (node != null && node.getUuid() == null) {
          parent = createSpace(parent, unitPath);
        }
      }

    } catch (Exception e) {
    	System.err.println("Error Creating the Alfresco Path");
      parent = null;
    }
    return parent;
  }
  
  public List<AlfrescoFile> getContent(String path){
	  List<AlfrescoFile> retList = null;
	  Reference ref = null;
	    try {
	      startSession();
	      ref = createPathToAlfrescoNoSession(path);
	      if (ref != null)
	        retList = queryChildren(ref);
	    } catch (Exception e) {
	      System.err.print("Error Loading Content Space: "+e.getLocalizedMessage());
	    }
	    endSession();
	    ref=null;
	    return retList;
  }
  
  public AlfrescoFile getContent (String path, String fileName){
	  AlfrescoFile afile = new AlfrescoFile();
	  Reference ref = refExists(path, fileName);
	  afile.setFileName(fileName);
	    if (ref!=null) {
	      try {
	        startSession();
        	afile.setPath(ref.getPath());
        	afile.setStore(ref.getStore());
        	afile.setUuid(ref.getUuid());
        	afile.setFileData(getContentAsByte(ref));
	      } catch (Exception e) {
	    	  System.err.println("Error getting the content of the file: "+ e.getLocalizedMessage());
	      }
	      endSession();
	    }

	    return afile;
	  
  }
  
  public List<String[]> getProperties(String path,String fileName){
	  List<String[]> props = new ArrayList<String[]>();
	  Reference ref = refExists(path, fileName);
	    if (ref!=null) {
	    	startSession();
	    	props = getProperties(ref);
	    	endSession();
	    }
	    return props;
  }
  
  public boolean fileExists(String path, String fileName) {
    boolean exists = false;
    try {
      startSession();
      Reference ref = createPathToAlfrescoNoSession(path);
      if (ref != null)
        if (getContent(ref, fileName) != null)
          exists = true;
    } catch (Exception e) {
    	System.err.println("Error getting the content of the file: "+ e.getLocalizedMessage());
    }
    endSession();
    return exists;
  }
  
  private Reference refExists(String path, String fileName) {
	  Reference exists = null;
	    try {
	      startSession();
	      Reference ref = createPathToAlfrescoNoSession(path);
	      if (ref != null)
	           exists = getContent(ref, fileName);
	    } catch (Exception e) {
	    	System.err.println("Error getting the Space reference: "+ e.getLocalizedMessage());
	    }
	    endSession();
	    return exists;
	  }

  public byte[] readFile(String path, String fileName) {
    byte[] data = null;
    if (fileExists(path, fileName)) {
      try {
        startSession();
        Reference ref = createPathToAlfrescoNoSession(path);
        if (ref != null)
          data = getContentAsByte(ref, fileName);
      } catch (Exception e) {
    	  System.err.println("Error reading the file: "+ e.getLocalizedMessage());
      }
      endSession();
    }

    return data;
  }

  @SuppressWarnings("unused")
private Reference uploadFileToAlfresco(Reference spaceName, String name, String pathToFile,
      Aspect as) {
    Reference doc = null;

    startSession();
    byte[] data = fileToByte(pathToFile);
    try {

      doc = getContent(spaceName, name);
      if (doc == null)
        doc = createDocument(spaceName, name, data, as);
      else
        doc = updateDocument(doc, name, data, as);
    } catch (Exception e) {
    	System.err.println("Uploading the Alfresco to the file: "+ e.getLocalizedMessage());
    }
    endSession();

    return doc;
  }
  
  public Reference updateBytesToAlfresco(String path, String name, byte[] data, String description){
	  Reference doc = null;	
	  startSession();
	  String alfresco_path[] = path.split(SEPARATOR);
	  Reference parent = null;
	  try {
	    parent = getCompanyHome();
	    String currentNode = "";
	    for (String unitPath : alfresco_path) {
	      currentNode = currentNode + SEPARATOR + unitPath;
	      Reference node = getSpaceReference(currentNode);
	      if (node != null && node.getUuid() == null) {
	        parent = createSpace(parent, unitPath);
	      }
	    }
	    if (parent != null) {
	    	doc = getContent(parent, name);
		    if (doc == null)
		      doc = createDocument(parent, name, data);
		    else
		      doc = updateDocument(doc, name, data, description);
	    }
	  } catch (Exception e) {
	    System.err.println("Error uploading the content: "+ e.getLocalizedMessage());
	    parent = null;
	  }
	  endSession();
	  return doc;
  }

  public Reference uploadBytesToAlfresco(String path, String name, byte[] data) {
	    Reference doc = null;

	    startSession();
	    String alfresco_path[] = path.split(SEPARATOR);
	    Reference parent = null;
	    try {
	      parent = getCompanyHome();
	      String currentNode = "";
	      for (String unitPath : alfresco_path) {
	        currentNode = currentNode + SEPARATOR + unitPath;
	        Reference node = getSpaceReference(currentNode);
	        if (node != null && node.getUuid() == null) {
	          parent = createSpace(parent, unitPath);
	        }
	      }
	      if (parent != null) {
	        doc = createDocument(parent, name, data);
	      }
	    } catch (Exception e) {
	      System.err.println("Error uploading the content: "+ e.getLocalizedMessage());
	      parent = null;
	    }
	    endSession();
	    return doc;
	  }

  
  public boolean deleteDocument(AlfrescoFile aFile){
	  boolean ret = false;
	  startSession();
	  try {
		  if (aFile != null) {
	    	  deleteSpace(aFile);
	    	  ret = true;
		  }
	  } catch (Exception e) {
		  System.err.println("Error Deleting the Document: "+ e.getLocalizedMessage());
	  }
	  endSession();
	  
	  return ret;
  }
  
  public boolean deleteSpace(String path){
	  boolean ret = false;
	  startSession();
	  String alfresco_path[] = path.split(SEPARATOR);
	  Reference parent = null;
	  try {
	      parent = getCompanyHome();
	      String currentNode = "";
	      for (String unitPath : alfresco_path) {
	        currentNode = currentNode + SEPARATOR + unitPath;
	        Reference node = getSpaceReference(currentNode);
	        if (node != null && node.getUuid() == null) {
	          parent = createSpace(parent, unitPath);
	        }
	      }
	      if (parent != null) {
	        deleteSpace(parent);
	        ret = true;
	      }
	  } catch (Exception e) {
		  System.err.println("Error Deleting the Space: "+ e.getLocalizedMessage());
	     parent = null;
	  }
	  endSession();
	  return ret;
  }
  
  @Deprecated
  /**
   * @deprecated Current is uploadBytesToAlfresco
   * */
  public boolean uploadFileToAlfresco(String path, String name, byte[] data) {
    boolean res = false;
    startSession();
    String alfresco_path[] = path.split(SEPARATOR);
    Reference parent = null;
    try {
      parent = getCompanyHome();
      String currentNode = "";
      for (String unitPath : alfresco_path) {
        currentNode = currentNode + SEPARATOR + unitPath;
        Reference node = getSpaceReference(currentNode);
        if (node != null && node.getUuid() == null) {
          parent = createSpace(parent, unitPath);
        }
      }
      if (parent != null) {
        if (createDocument(parent, name, data) != null)
          res = true;
      }
    } catch (Exception e) {
      e.printStackTrace();
      parent = null;
    }
    endSession();
    return res;
  }

  private byte[] fileToByte(String pathToFile) {
    File f = new File(pathToFile);
    FileInputStream fin = null;
    FileChannel ch = null;
    byte[] bytes = null;
    try {
      fin = new FileInputStream(f);
      ch = fin.getChannel();
      int size = (int) ch.size();
      MappedByteBuffer buf = ch.map(MapMode.READ_ONLY, 0, size);
      bytes = new byte[size];
      buf.get(bytes);

    } catch (IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    } finally {
      try {
        if (fin != null) {
          fin.close();
        }
        if (ch != null) {
          ch.close();
        }
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
    return bytes;

  }

}