package org.adempiere.webui.component;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import org.adempiere.webui.panel.WAttachment;
import org.adempiere.webui.theme.ThemeManager;
import org.compiere.model.MAttachment;
import org.compiere.model.MAttachmentECMEntry;
import org.compiere.model.MAttachmentEntry;
import org.compiere.util.CLogger;
import org.compiere.util.Env;
import org.compiere.util.Msg;
import org.compiere.util.Util;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.event.Events;
import org.zkoss.zk.ui.event.UploadEvent;
import org.zkoss.zul.Cell;
import org.zkoss.zul.Hbox;
import org.zkoss.zul.Menuitem;
import org.zkoss.zul.North;
import org.zkoss.zul.Popup;
import org.zkoss.zul.South;
import org.zkoss.zul.Vlayout;
import org.zkoss.zul.West;

import com.bmlaurus.alfresco.AlfrescoUpload;
import com.bmlaurus.alfresco.model.MALFileList;

public class FileItem extends ListItem implements EventListener<Event>{

	/**
	 * 
	 */
	private static final long serialVersionUID = 5809221638933993540L;
	private final CLogger log = CLogger.getCLogger(getClass());
	
	public static final String EVENT_ATTRIBUTE = "EVENT";
	public static final String PROP_EVENT = "PROPERTIES";
	public static final String ADD_EVENT = "ADD_UPDATE";
	private static final String BUILD_ATTRIBUTE = "BUID_ECM";
	private static final String LISTBOX_ATTRIBUTE = "LISTBOX_ECM";

	private Checkbox item;
	private String name;
	private MALFileList mfile;
	private MAttachment m_attachment;
	private WAttachment w_attachment;
	private AlfrescoUpload alfUp;
	private String adp;
	private int fileIndex;
	
	private static final String[] lISTENER_EVENTS = {Events.ON_CLICK, Events.ON_UPLOAD};
	
	public FileItem(MALFileList _file, MAttachment attachment, WAttachment wattachment, AlfrescoUpload _alfUp){
		mfile=_file;
		alfUp=_alfUp;
		name=mfile.getName();
		m_attachment = attachment;
		w_attachment = wattachment;
		if(w_attachment!=null && w_attachment.getAttribute(BUILD_ATTRIBUTE)==null)
			fileListVersion();
		
		if(_file.getDescription() != null)
			this.setTooltiptext(_file.getDescription());

		item = new Checkbox();
		item.setText(name);
		item.setEnabled(false);
		
		ListCell cell3 = new ListCell();
		cell3.appendChild(item);
		initPopup(cell3);
		this.appendChild(cell3);
		
		for (String event : lISTENER_EVENTS)
        {
            this.addEventListener(event, this);
        }
		log.info("Created new File Item");
	}
	
	public MALFileList getMFile(){
		return mfile;
	}
	
	public String getName(){
		return name;
	}

	public void setChecked(boolean _checked){
		item.setChecked(_checked);
	}
	
	public boolean getChecked(){
		return item.isChecked();
	}

	public void setAdp(String adPath){
		adp=adPath;
	}
	
	public void setFileIndex(int index){
		fileIndex=index;
	}
	
	private void preview(){
		Method method;
		try {
			method = w_attachment.getClass().getDeclaredMethod("clearPreview");
			method.setAccessible(true);
			method.invoke(w_attachment);
			method.setAccessible(false);
			
			Field field = w_attachment.getClass().getDeclaredField("cbContent");
			field.setAccessible(true);
			if(field.get(w_attachment) instanceof Listbox)
			{
				Listbox cbContent = (Listbox) field.get(w_attachment);
				cbContent.setSelectedIndex(fileIndex);
			}
			field.setAccessible(false);
			
			method = w_attachment.getClass().getDeclaredMethod("autoPreview",int.class,boolean.class);
			method.setAccessible(true);
			method.invoke(w_attachment,fileIndex,false);
			method.setAccessible(false);
			
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void initPopup(ListCell cell){
		final Menupopup mnu = new Menupopup();
		
		Menuitem properties = new Menuitem();
		properties.setAttribute(EVENT_ATTRIBUTE, PROP_EVENT);
		properties.setLabel(Util.cleanAmp(Msg.getMsg(Env.getCtx(), "Properties")));
		properties.setImage(ThemeManager.getThemeResource("/images/Wizard24.png"));
        properties.addEventListener(Events.ON_CLICK, this);
        mnu.appendChild(properties);
        
        Menuitem addUpdate = new Menuitem();
        addUpdate.setAttribute(EVENT_ATTRIBUTE, ADD_EVENT);
        addUpdate.setLabel(Util.cleanAmp(Msg.getMsg(Env.getCtx(), "Add/Update")));
        addUpdate.setImage(ThemeManager.getThemeResource("/images/Import24.png"));
        addUpdate.setUpload("true");
        addUpdate.addEventListener(Events.ON_UPLOAD, this);
        mnu.appendChild(addUpdate);
        
        this.addEventListener(Events.ON_RIGHT_CLICK, new EventListener<Event>() {
        	@Override
        	public void onEvent(Event event) throws Exception {
	        	mnu.open(event.getTarget());	
        	}
		});
        
        this.addEventListener(Events.ON_MOUSE_OUT, new EventListener<Event>() {
        	@Override
        	public void onEvent(Event event) throws Exception {
        		mnu.close();	
        	}
		});
        
        cell.appendChild(mnu);
	}
	
	public void createPropertiesWindow(Component ref,List<String[]> properties){
		Popup prop = new Popup();
		ref.getParent().getParent().appendChild(prop);
		Vlayout panel = new Vlayout();
		//Title Label
		Label lbl = new Label(getMFile().getName() + " Properties");
		lbl.setStyle("font-size: 14px; font-weight: bold;");
		panel.appendChild(lbl);
		//Properties Grid
		Grid grid = new Grid();
		//Grid Rows
		Rows rows = new Rows();
		grid.appendChild(rows);
		for (String[] property:properties){
			Row row = new Row();
			Cell cellLabel = new Cell();
			cellLabel.appendChild(new Label(property[0]));
			row.appendChild(cellLabel);
			Cell cellValue = new Cell();
			cellValue.appendChild(new Label(property[1]));
			row.appendChild(cellValue);
			rows.appendChild(row);
		}
		panel.setWidth("450px");
		panel.appendChild(grid);
		prop.appendChild(panel);
		prop.open(ref.getParent().getParent(),"overlap");

	}
	
	public void appendFile(){
		Listbox list = getListBox();
		if(list!=null)
			list.appendChild(this);
	}
	
	private Listbox getListBox(){
		Listbox list = null;
		if(w_attachment!=null)
			list = (Listbox) getComponentChild(w_attachment);
		
		return list;
	}
	
	private Component getComponentChild(Component c){
		Component compRet = null;
		for(Component comp : c.getChildren()){
			if(comp.getAttribute(LISTBOX_ATTRIBUTE)==null)
			{
				if(comp.getChildren().size()>0)
				{
					compRet = getComponentChild(comp);
					if(compRet != null && 
							compRet.getAttribute(LISTBOX_ATTRIBUTE) != null && 
							compRet.getAttribute(LISTBOX_ATTRIBUTE).equals(LISTBOX_ATTRIBUTE))
						return compRet;
					if(comp.getAttribute(LISTBOX_ATTRIBUTE) == null)
						compRet=null;
				}
			}else
				return comp;
		}
		return compRet;
	}
	
	@Override
	public void onEvent(Event event) throws Exception {
		if (event instanceof UploadEvent)
		{
			w_attachment.onEvent(event);
			//Take Last entry
			MAttachmentEntry entry = null;
			byte[] data = null;
			//review items for updates.
			fileIndex = m_attachment.m_items.size()-1;
			data = m_attachment.getEntryData(m_attachment.m_items.size()-1);
			for(int i=0;i<m_attachment.m_items.size();i++){
				if(m_attachment.getEntry(i).getName().equals(getMFile().getFileName()))
					fileIndex = i;
			}
			if(fileIndex!=m_attachment.m_items.size()-1)
				m_attachment.m_items.remove(m_attachment.m_items.size()-1);
			
			entry = m_attachment.getEntry(fileIndex);

			MAttachmentECMEntry alfItem = null;
			if(entry instanceof MAttachmentECMEntry){
				alfItem = (MAttachmentECMEntry) entry;
				alfItem.setData(data);
			}
			else
				alfItem = new MAttachmentECMEntry(getMFile().getFileName(),data,entry.getIndex(), null);
			if(alfItem.getAlf_File()==null)
				alfItem.setMode(MAttachmentECMEntry.MODE_NEW);
			else
				alfItem.setMode(MAttachmentECMEntry.MODE_UPDATE);
			m_attachment.m_items.set(fileIndex, alfItem);
			setChecked(true);
			preview();
			log.info("Adding new attachment entry: "+ getMFile().getName());
		} else if(event.getTarget()!=null && 
				event.getTarget().getAttribute(FileItem.EVENT_ATTRIBUTE)!= null && 
				event.getTarget().getAttribute(FileItem.EVENT_ATTRIBUTE).equals(FileItem.PROP_EVENT)){
			createPropertiesWindow(event.getTarget(),alfUp.getProperties(adp, getMFile().getFileName()));
		} else if(event.getName().equals(Events.ON_CLICK)){
			preview();
		}
	}	
	
	
	private void fileListVersion(){
		w_attachment.setWidth("800px");
		org.zkoss.zul.Borderlayout m_main_panel = (org.zkoss.zul.Borderlayout) w_attachment.getChildren().get(0);
		
		West westPanel = new West();
		westPanel.setCollapsible(false);
		westPanel.setSplittable(true);
		
		Listbox listBox = new Listbox();
		listBox.setAttribute(LISTBOX_ATTRIBUTE, LISTBOX_ATTRIBUTE);
		Vlayout listPanel = new Vlayout();
		westPanel.appendChild(listPanel);
		listPanel.appendChild(listBox);
				
		listPanel.setHeight("100%");
		listPanel.setWidth("150px");
		listPanel.setVisible(false);
		m_main_panel.appendChild(westPanel);
		
		if(w_attachment instanceof WAttachment)
		{
			listPanel.setVisible(true);
			
			for(Component comp : m_main_panel.getChildren()){
				//look for first Hbox, and drop the delete button.
				if(comp instanceof North){
					for(Component cdiv : comp.getChildren()){
						Component c = cdiv.getChildren().get(0);
						if(c instanceof Hbox){
							for(Component butt : c.getChildren()){
								if(butt instanceof Button){
									if(((Button)butt).getImage().endsWith("/images/Delete24.png") ||
											((Button)butt).getImage().endsWith("/images/Import24.png"))
										((Button)butt).setVisible(false);
								}else if(butt instanceof Listbox)
									butt.setVisible(false);
								else if(butt instanceof Label)
									butt.setVisible(false);
							}
						}
					}
				} else if(comp instanceof South){
					for(Component c : comp.getChildren()){
						if(c instanceof Hbox){
							for(Component butt : c.getChildren()){
								if(butt instanceof Button){
									if(((Button)butt).getImage().endsWith("/images/Delete24.png"))
										((Button)butt).setEnabled(false);
								}
							}
						}
					}
				}
			}
		}
		w_attachment.setAttribute(BUILD_ATTRIBUTE, BUILD_ATTRIBUTE);
		log.info("Rebuilding Window for ECM");
	}
}
