package org.compiere.model;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;

import org.adempiere.model.GenericPO;
import org.adempiere.webui.panel.WAttachment;
import org.alfresco.webservice.types.Reference;
import org.compiere.util.CLogger;
import org.compiere.util.Env;
import org.adempiere.webui.component.FileItem;

import com.bmlaurus.alfresco.AlfrescoFile;
import com.bmlaurus.alfresco.AlfrescoUpload;
import com.bmlaurus.alfresco.model.MALAlfrescoModel;
import com.bmlaurus.alfresco.model.MALFileList;
import com.bmlaurus.alfresco.model.X_AL_Alfresco_Schema;
import com.bmlaurus.alfresco.model.MAttachment;

/**
 * @author Antonio Caaveral
 * @company BMLaurus
 * */
public class AttachmentECMSystem implements IAttachmentStore {

	private MALAlfrescoModel alfrescoModel;
	
	private AlfrescoUpload alfUp;

	private WAttachment attachment;
	
	private final CLogger log = CLogger.getCLogger(getClass());
	/** ECM Integration (Alfresco Integration)*/
	public static final String ECM = "ecm";
	
	@Override
	public boolean loadLOBData(org.compiere.model.MAttachment attach, MStorageProvider prov) {
		boolean ret = false;
		//		Reset
		attach.m_items = new ArrayList<MAttachmentEntry>();
		
		validateAlfrescoModel((MAttachment) attach);
		
		if(alfrescoModel!=null && alfrescoModel.getType().equals(MALAlfrescoModel.TYPE_FileUpload))
		{
			String adp = parsePath((MAttachment) attach,alfrescoModel.getAlfrescoDestinationPath());
			List<AlfrescoFile> content = alfUp.getContent(adp);
			if(content!=null){
				for(AlfrescoFile alfFile : content){
					String fileName = alfFile.getFileName();
					if(fileName!=null){
						byte[] dataEntry = alfFile.getFileData();
						attach.m_items.add (new MAttachmentECMEntry (fileName, dataEntry, attach.m_items.size()+1,alfFile));
					}
				}
				ret=true;
			}
		}else if (alfrescoModel!=null && alfrescoModel.getType().equals(MALAlfrescoModel.TYPE_FileList)){
			List<MALFileList> flist = alfrescoModel.getFileList();
			if(flist!=null && flist.size()>0){
				for(MALFileList _file:flist){
					
					String path = alfrescoModel.getAlfrescoDestinationPath();
					if(_file.getAlfrescoDestinationPath()!=null)
						path=_file.getAlfrescoDestinationPath();
					String adp = parsePath((MAttachment) attach, path);
					AlfrescoFile alfFile = alfUp.getContent(adp, _file.getFileName());
					
					if(((MAttachment) attach).getStorageWindow() instanceof WAttachment)
						attachment = (WAttachment) ((MAttachment) attach).getStorageWindow();
					FileItem chkFile = new FileItem(_file,attach,attachment,alfUp);
					chkFile.setAdp(adp);
					
					if(alfFile.getUuid()!=null){
						MAttachmentECMEntry entry = new MAttachmentECMEntry (alfFile.getFileName(), alfFile.getFileData(),attach.m_items.size()+1,alfFile);
						chkFile.setFileIndex(attach.m_items.size());
						attach.m_items.add (entry);
						chkFile.setChecked(true);
					}
					
					chkFile.appendFile();
				}
			}
		}
		return ret;
	}

	@Override
	public boolean save(org.compiere.model.MAttachment attach, MStorageProvider prov) {
		boolean ret=false;
		if (attach.m_items == null || attach.m_items.size() == 0)
		{
			setAlfrescoValues((MAttachment) attach,null);
			return true;
		}
		
		try {
			validateAlfrescoModel((MAttachment) attach);
			
			StringBuffer values = new StringBuffer();
			String adp = parsePath((MAttachment) attach,alfrescoModel.getAlfrescoDestinationPath());
			for (int i = 0; i < attach.m_items.size(); i++)
			{
				MAttachmentEntry item = attach.getEntry(i);
				MAttachmentECMEntry alfItem = null;
				if(item instanceof MAttachmentECMEntry)
				{
					alfItem = (MAttachmentECMEntry) item;
				}else
					alfItem = new MAttachmentECMEntry(item.getName(), item.getData(),item.getIndex(), null);
				if(alfItem.getAlf_File()==null || alfItem.getMode() == MAttachmentECMEntry.MODE_UPDATE)
				{

					String desc = "user: "+MUser.getNameOfUser(Env.getAD_User_ID(Env.getCtx()));
					Reference ref = alfUp.updateBytesToAlfresco(adp, alfItem.getName(), alfItem.getData(), desc);
					AlfrescoFile afile = new AlfrescoFile();
					afile.setFileData(alfItem.getData());
					afile.setFileName(alfItem.getName());
					afile.setUuid(ref.getUuid());
					afile.setPath(ref.getPath());
					afile.setStore(ref.getStore());
					alfItem.setAlf_File(afile);
				}
				values.append(alfItem.getAlf_File().toString());
				values.append(",");
			}
			setAlfrescoValues((MAttachment) attach,values.toString());
			ret=true;
			
		} catch (Exception e) {
			log.log(Level.SEVERE, "saveLOBDataToECM", e);
			setAlfrescoValues((MAttachment) attach,null);
		}
		return ret;
	}

	@Override
	public boolean delete(org.compiere.model.MAttachment attach, MStorageProvider prov) {
		
		if(attach.getEntryCount()>0){
			validateAlfrescoModel((MAttachment) attach);
			String path = parsePath((MAttachment) attach,alfrescoModel.getAlfrescoDestinationPath());
			alfUp.deleteSpace(path);
		}
	
		return true;
	}

	@Override
	public boolean deleteEntry(org.compiere.model.MAttachment mAttachment,
			MStorageProvider provider, int index) {
		boolean ret = false;
		if (index >= 0 && index < mAttachment.m_items.size()) {
			log.config("Index=" + index + " - NewSize=" + mAttachment.m_items.size());
			try {
				validateAlfrescoModel((MAttachment) mAttachment);
				MAttachmentEntry item = mAttachment.getEntry(index);
				MAttachmentECMEntry alfItem = null;
				if(item instanceof MAttachmentECMEntry)
				{
					alfItem = (MAttachmentECMEntry) item;
					ret = alfUp.deleteDocument(alfItem.getAlf_File());
					mAttachment.m_items.remove(index);
				}
				
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
		log.warning("Not deleted Index=" + index + " - Size=" + mAttachment.m_items.size());
		return ret;
	}
	
	
	
	/**
	 *  @author Antonio Caaveral
	 *  No Changes in M, X and I clases
	 *  Coma-Separated ECM IDs on Record.
	 * */
	public static final String COLUMNNAME_AlfrescoValues = "AlfrescoValues";
	private String AlfrescoValues;
	public void setAlfrescoValues(MAttachment attach, String _AlfrescoValues){
		attach.set_CustomColumn(COLUMNNAME_AlfrescoValues, _AlfrescoValues);
	}
	public String getAlfrescoValues(MAttachment attach){
		AlfrescoValues=attach.get_ValueAsString(COLUMNNAME_AlfrescoValues);
		return AlfrescoValues;
	}
	//End Changes in X and I clases
	
	
	/**
	 * Parse Alfresco Destination Path
	 * @author Antonio Caaveral
	 * @param realPath
	 * @returns Path Parsed
	 * */
	public static String parsePath(MAttachment attach,String realPath) {
	    realPath = realPath.replaceAll("#Year", (new Date()).getYear() + 1900 + "");
	    realPath = realPath.replaceAll("#Month", (new Date()).getMonth() + 1 + "");
	    realPath = realPath.replaceAll("#Day", (new Date()).getDay() + "");
	    MTable table = new MTable(Env.getCtx(), attach.getAD_Table_ID(), null);
	    realPath = realPath.replaceAll("#TableID", table.getAD_Table_ID()+"");
	    realPath = realPath.replaceAll("#TableName", table.getTableName());
	    realPath = realPath.replaceAll("#RecordID", attach.getRecord_ID()+"");
	    
	    String path = Env.parseVariable(realPath, new GenericPO(attach.getAD_Table().getTableName(), attach.getCtx(),
	    		attach.getRecord_ID()), attach.get_TrxName(), false);
	    return path;
	  }

	/**
	 * Static call of Alfresco Connection, for Version Control & Properties viewer
	 * */
	public static AlfrescoUpload getAlfrescoUpload(MAttachment attach){
		MTable table = MTable.get(Env.getCtx(), attach.getAD_Table_ID());
		int model = table.get_ValueAsInt("AL_Alfresco_Model_ID");
		if(model>0)
		{
			MALAlfrescoModel alfModel = new MALAlfrescoModel(attach.getCtx(), model, attach.get_TrxName());
			X_AL_Alfresco_Schema schema = new X_AL_Alfresco_Schema(Env.getCtx(), alfModel.getAL_Alfresco_Schema_ID(), null);
			String server_host = "localhost";
			String server_port = "80";
			if(schema.getHostAddress().indexOf(":")>0)
			{
				server_host = schema.getHostAddress().split(":")[0];
				server_port = schema.getHostAddress().split(":")[1];
			}else
				server_host = schema.getHostAddress();
			
			String server_username = schema.getUserName();
			String server_password = schema.getPassword();
			
			return new AlfrescoUpload(server_host,server_port,server_username,server_password);
		}
		return null;
	}
	
	private void initCMSConnector(MAttachment attach) {
		MTable table = MTable.get(Env.getCtx(), attach.getAD_Table_ID());
		int model = table.get_ValueAsInt("AL_Alfresco_Model_ID");
		if(model>0)
		{
			alfrescoModel = new MALAlfrescoModel(attach.getCtx(), model, attach.get_TrxName());
			attach.setTitle( "ECM - "+alfrescoModel.getName());
			X_AL_Alfresco_Schema schema = new X_AL_Alfresco_Schema(Env.getCtx(), alfrescoModel.getAL_Alfresco_Schema_ID(), null);
			String server_host = "localhost";
			String server_port = "80";
			if(schema.getHostAddress().indexOf(":")>0)
			{
				server_host = schema.getHostAddress().split(":")[0];
				server_port = schema.getHostAddress().split(":")[1];
			}else
				server_host = schema.getHostAddress();
			
			String server_username = schema.getUserName();
			String server_password = schema.getPassword();
			
			alfUp= new AlfrescoUpload(server_host,server_port,server_username,server_password);
		}
	}
	
	private void validateAlfrescoModel(MAttachment attach){
		if(alfrescoModel==null)
			initCMSConnector(attach);
		else{
			MTable table = MTable.get(Env.getCtx(), attach.getAD_Table_ID());
			int model = table.get_ValueAsInt("AL_Alfresco_Model_ID");
			if(alfrescoModel.getAL_Alfresco_Model_ID()!=model)
				initCMSConnector(attach);
			if(attach.getTitle()==null && table!=null)
				attach.setTitle(table.get_TableName()+"_"+attach.getRecord_ID());
		}
	}
	
}
